/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 0.81B */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "globals.h"
#include "proto.h"

void
set_global_black( )

	{
	PhRect_t	rect = { 0, 0, 1023, 767 };
	PgSetFillColor( Pg_BLACK );
	PgSetStrokeColor( Pg_BLACK );
	PgDrawRect( &rect, Pg_DRAW_FILL_STROKE );
	PgFlush();
	}

int 
set_global_palette( PgColor_t *PalBuf, int expose ) {
	int i;


#if 0
	if ( !expose ) {
		PalBuf+=2;	/* first two colours are assumed to be black and white */
		for (i=0; i<254; i++) if (*(PalBuf+i) == 0) break;
		PgSetPalette( PalBuf, 0, 255-i, i, Pg_PALSET_GLOBAL, 0 );
		}
	else {
		PgSetPalette( PalBuf, 0, 0, 256, Pg_PALSET_GLOBAL, 0 );
		}
#else
	PalBuf+=2;	/* first two colours are assumed to be black and white */
	if ( force_expose )
		PgSetPalette( PalBuf, 0, 2, 254, Pg_PALSET_GLOBAL | Pg_PALSET_FORCE_EXPOSE, 0 );
	else
		PgSetPalette( PalBuf, 0, 2, 254, Pg_PALSET_GLOBAL, 0 );
#endif
	PalBuf-=2;
	memcpy( prev_pal, PalBuf, 256 );
	PgFlush();
	return( 0 );
}

int
filesel_open( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	int						i, pal_ind;
	char					path[PATH_MAX], buffer[256];
	FILE					*fp;
	AwFileSelectCallback_t	*fs = cbinfo->cbdata;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/* free memory in use */
	for ( i = 0; i < slide_ctr; i++ ) {
		free( slides[i] );
		if ( pals[i] )
			free( pals[i] );
		if ( images[i] ) {
			if ( images[i]->palette )
				free( images[i]->palette );
			if ( images[i]->image )
				free( images[i]->image );
			free( images[i] );
			}
		}

	if ( slide_ctr ) {
		free( slides );
		free( images );
		free( pals );
		slides = NULL;
		images = NULL;
		pals = NULL;
		slide_ctr = 0;
		}

	/* save path */
	strcpy( slide_path, fs->dirpath );

	/* open new file */
	sprintf( path, "%s/%s", fs->dirpath, fs->filename );
	if ( NULL == ( fp = fopen( path, "r" ) ) ) {
		return( Pt_END );
		}

	/* read list of slides */
	pal_ind = 0;
	while( NULL != fgets( buffer, 256, fp ) ) {
		/* skip comments and blank lines */
		if ( buffer[0] == '#' || buffer[0] == ' ' || buffer[0] == '\n' )
			continue;
		/* remove newline char */
		ApNLErase( buffer );
		if ( strstr( buffer, "pal" ) ) {
			pals = realloc( pals, ( slide_ctr + 1 ) * sizeof( char * ) );
			pals[slide_ctr] = strdup( buffer );
			pal_ind = 1;
			continue;
			}
		if ( !pal_ind ) {
			pals = realloc( pals, ( slide_ctr + 1 ) * sizeof( char * ) );
			pals[slide_ctr] = NULL;
			}
		slides = realloc( slides, ( slide_ctr + 1 ) * sizeof( char * ) );
		slides[slide_ctr] = strdup( buffer );
		images = realloc( images, ( slide_ctr + 1 ) * sizeof( PhImage_t * ) );
		images[slide_ctr] = NULL;
		slide_ctr++;
		pal_ind = 0;
		}

	fclose( fp );

	/* display first slide */
	if ( slide_ctr > 0 ) {

		/* read slides */
		screen_read( 0 );
		screen_update( ABW_sscreen, 0, 0 );
		if ( slide_ctr > 1 )
			screen_read( 1 );
		
		/* set slide position to start */
		slide_curr = 0;
		first_time = 1;
		}
	else {
		ApMessageDialog( ABW_base, NULL, "Slide file is empty. Try another file.", "helv14.phf", " Ok " );
		slide_curr = -1;
		}


	return( Pt_CONTINUE );
	}

int
palette_update( char *palpath, PhImage_t *image )

	{
	int			expose = 0;
	FILE		*fp;
	char		buffer[1024];
	char		path[PATH_MAX];

	old_gc = PgSetGC( gc );
	PgSetRegion( Ph_DEV_RID );

	if ( palpath == default_pal )
		expose = Pg_PALSET_FORCE_EXPOSE;

	/* if "pal" - treat as palette file */
	if ( !palpath ) {
		if ( memcmp( prev_pal, image->palette, 256 ) ) {
			set_global_black( );
			set_global_palette( image->palette, expose );
			}
		PgSetGC( old_gc );
		return( 0 );
		}
		
	/* resolve path offset */
	if ( palpath[0] == '/' )
		strcpy( path, palpath );
	else {
		strcpy( path, slide_path );
		strcat( path, "/" );
		strcat( path, palpath );
		}
	
	if ( fp = fopen( path, "r" ) ) {
		fread( &buffer, 1024, 1, fp ); 
		set_global_palette( (PgColor_t *) buffer, expose );	
		fclose( fp );
		}

	PgSetGC( old_gc );
	return( 0 );
	}
